/*
 *
 * $Id: wrcmos.c,v 1.1 1998/09/24 21:08:20 hendriks Exp $
 */
#include <stdio.h>
#include <unistd.h>
#include <errno.h>
#include <asm/io.h>

static inline
unsigned char readreg(int regno) {
  outb(regno, 0x70);
  return inb(0x71);
}

static inline
void writereg(int regno, unsigned char val) {
  outb(regno, 0x70);
  outb(val, 0x71);
}

void usage()
{
  fprintf(stderr, "Usage: wrcmosbits [-d]\n");
  fprintf(stderr, "Reads a bitindex: bitvalue stream from stdin\n");
  fprintf(stderr, "-d will just dump the settings for what it would do\n");
  exit(1);
}
unsigned char data[128];
unsigned int bits_seen[8*128];


// operating assumption: any bits not set in this input stream 
// are bits we don't care about. So we will zero out the data, read in 
// the set/clear bits, then write the data out. 
int main(int argc, char *argv[]) {
  int i;
  int do_not_do_this = 0;
  int bitvalue, bitindex, byteindex, bitnumber;
  int debug = 0;

  if (argc > 1) {
    if (! strcmp(argv[1], "-d"))
      debug = 1;
    else
      usage();
  }

  if (! debug) {
    if (ioperm(0x70, 2, 1) == -1) {
      perror("ioperm");
      exit(1);
    }
  }


  memset(data, 0, sizeof(data));
  memset(bits_seen, 0, sizeof(bits_seen));
  while (scanf("%d: %d\n", &bitnumber, &bitvalue) == 2) {
    if (bits_seen[bitnumber]) {
      bits_seen[bitnumber]++;
      printf("WARNING: bit %d seen %d times\n", 
	     bitnumber, bits_seen[bitnumber]);
      do_not_do_this = 1;
      printf("CMOS WILL NOT BE SET -- FIX YOUR BITS FILE\n");
      continue;
    }
    else
      bits_seen[bitnumber]++;

    // numbering: bit 0 is byte 0, LSB
    byteindex = bitnumber >> 3;
    bitindex = bitnumber & 7;

    // we only care if it says to set it, since we start out all 0
    if (bitvalue)
      data[byteindex] |= (bitvalue << bitindex);
  }

  if ((! debug) && (!do_not_do_this))
    for (i=0; i < 128; i++) writereg(i, data[i]);
  if ( debug) {
    int i, j;
    /* now dump the bits in bit #,value format */
    for(i = 0; i < 128; i++) {
      unsigned char byte = data[i];
      int j;
      //      printf("(%d: 0x%02x): ", i, byte);
      for(j = 0; j < 8; j++) {
	int bitnumber = i*8 + j;
	// bit 0 is byte 0 LSB, and so forth.
	int bitvalue = byte & (1<<j) ? 1 : 0;
	printf("%d: %d\n", bitnumber, bitvalue);
      }
      //printf("\n");
    }
  }

  exit(0);
}


